@@ -8,6 +8,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdarg.h>
 #include <errno.h>
 #include <ctype.h>
 #include <time.h>
@@ -19,6 +20,7 @@
 #include "in_formats.h"
 #include "out_formats.h"
 
+
 /*
  * Misc globals
  */
@@ -36,6 +38,9 @@ char	*configFileNames[SET_MAX_INFILES];
 char	*progName = NULL;
 
 
+/*
+ * Options and help
+ */
 t_arg argList[] = {
 	{ "?",	"help",		"Show this help", 0 },
 	{ "o",	"output",	"Specify output file (default stdout)", 1 },
@@ -44,12 +49,34 @@ t_arg argList[] = {
 	{ "of", "output-format","Specify output format", 1 },
 	{ "F",	"show-formats",	"Show list of predefined log- and output-formats", 0 },
 	{ "c",	"config",	"Specify configuration file (default: none)", 1 },
-	{ "q",	"quiet",	"Be less noisy. Good if you run from scripts", 0 }
+	{ "q",	"quiet",	"Use multiple times for more silence.", 0 }
 };
 
 const int argListN = (sizeof(argList) / sizeof(t_arg));
 
 
+void FERR(const char *pcFormat, ...)
+{
+ va_list ap;
+ va_start(ap, pcFormat);
+ fprintf(stderr, RA_NAME ": ");
+ vfprintf(stderr, pcFormat, ap);
+ va_end(ap);
+}
+
+
+void NICKDEBUG(const char *pcFormat, ...)
+{
+/*
+ va_list ap;
+ va_start(ap, pcFormat);
+ fprintf(stderr, "NICKDEBUG: ");
+ vfprintf(stderr, pcFormat, ap);
+ va_end(ap);
+*/
+}
+
+
 void showHelp()
 {
  fprintf(stderr, "\n" RA_NAME " " RA_VERSION " " RA_COPYRIGHT "\n");
@@ -73,7 +100,7 @@ void handleOpt(const int optN, char *optArg, char *currArg)
  	if (optArg)
  		destFileName = optArg;
  		else {
- 		fprintf(stderr, RA_NAME ": No output filename specified!\n");
+ 		FERR("No output filename specified!\n");
  		exit(2);
  		}
 	break;
@@ -83,7 +110,7 @@ void handleOpt(const int optN, char *optArg, char *currArg)
  	if (optArg)
  		userFileName = optArg;
  		else {
- 		fprintf(stderr, RA_NAME ": No user-file filename specified!\n");
+ 		FERR("No user-file filename specified!\n");
  		exit(2);
  		}
 	break;
@@ -107,13 +134,13 @@ void handleOpt(const int optN, char *optArg, char *currArg)
  		/* Check */
  		if (!isFound)
  			{
- 			fprintf(stderr, RA_NAME ": Invalid input (log-file) format '%s'\n", optArg);
+ 			FERR("Invalid input (log-file) format '%s'\n", optArg);
  			exit(2);
  			}
  		
  		setCurrInputFormat = i;
  		} else {
- 		fprintf(stderr, RA_NAME ": No input (log-file) format specified!\n");
+ 		FERR("No input (log-file) format specified!\n");
  		exit(2);
  		}
 	break;
@@ -136,13 +163,13 @@ void handleOpt(const int optN, char *optArg, char *currArg)
  		/* Check */
  		if (!isFound)
  			{
- 			fprintf(stderr, RA_NAME ": Invalid output format '%s'\n", optArg);
+ 			FERR("Invalid output format '%s'\n", optArg);
  			exit(2);
  			}
 
  		setOutputFormat = i;
  		} else {
- 		fprintf(stderr, RA_NAME ": No output format specified!\n");
+ 		FERR("No output format specified!\n");
  		exit(2);
  		}
  	break;
@@ -157,7 +184,7 @@ void handleOpt(const int optN, char *optArg, char *currArg)
  			inputFormats[i].ifDescription,
  			(i == 0) ? "(default)" : "");
  		}
- 		
+ 
  	fprintf(stderr, "\n" RA_NAME ": Available OUTPUT formats:\n");
  	for (i = 0; i < nOutputFormats; i++)
  		{
@@ -181,7 +208,7 @@ void handleOpt(const int optN, char *optArg, char *currArg)
 			nconfigFileNames++;
 			}
  		} else {
- 		fprintf(stderr, RA_NAME ": No configuration filename specified!\n");
+ 		FERR("No configuration filename specified!\n");
  		exit(2);
  		}
 	break;
@@ -193,7 +220,7 @@ void handleOpt(const int optN, char *optArg, char *currArg)
  	
  default:
  	/* Error */
- 	fprintf(stderr, RA_NAME ": Unknown argument '%s'.\n", currArg);
+ 	FERR("Unknown argument '%s'.\n", currArg);
  	break;
  }
 } 	
@@ -216,10 +243,9 @@ void handleFile(char *currArg)
  */
 int parse_int(char *inLine, int iLen, int *linePos)
 {
- int iResult;
- 
- iResult = 0;
- while (isdigit(inLine[*linePos]) && (iLen--))
+ int iResult = 0;
+
+ while (th_isdigit(inLine[*linePos]) && (iLen--))
  	{
  	iResult *= 10;
  	iResult += (inLine[(*linePos)++] - '0');
@@ -229,10 +255,8 @@ int parse_int(char *inLine, int iLen, int *linePos)
 }
 
 
-t_user_entry *parse_newuser(t_stats *pStats, char *newNick, BOOL autoAlias)
+t_user_entry *parse_newuser(t_stats *pStats, char *newNick)
 {
- int i, j;
- char tmpStr[SET_MAX_NICKLEN + 2 + 1];
  t_user_entry *tmpUser;
  t_nick_entry *tmpNick;
  
@@ -242,31 +266,18 @@ t_user_entry *parse_newuser(t_stats *pStats, char *newNick, BOOL autoAlias)
 	{
 	/* No, we need to create a new one */
 	tmpUser = user_new(newNick);
-
-	if (autoAlias)
-		{
-		/* Using auto-aliasing, strip special chars */
-		i = j = 0;
-		tmpStr[j++] = '*';
-		
-		while (newNick[i])
-			{
-			if (isalnum(newNick[i]))
-				tmpStr[j++] = newNick[i];
-			i++;
-			}
-		
-		tmpStr[j++] = '*';			
-		tmpStr[j++] = 0;
-					
-		/* Add "*strippednick*" */
-		tmpNick = nick_new(tmpStr);
-		} else
-		/* No auto-aliasing, normal nick */
-		tmpNick = nick_new(newNick);
+	tmpNick = nick_new(newNick);
 
 	tmpNick->user = tmpUser;
-	nick_insert(pStats->nickList, tmpNick);
+	if (nick_insert(pStats->nickList, tmpNick) != 0)
+		{
+		/* Failed, due to hash */
+		FERR("nick_insert() failed, hash: '%s'\n", newNick);
+		user_free(tmpUser);
+		nick_free(tmpNick);
+		return NULL;
+		}
+
 	user_insert(&pStats->userList, tmpUser);
 	}
 
@@ -274,99 +285,150 @@ t_user_entry *parse_newuser(t_stats *pStats, char *newNick, BOOL autoAlias)
 }
 
 
-
-int parse_public(char *inLine, char *fmt, t_stats *pStats, BOOL autoAlias)
+int parse_generic(char *inLine, char *fmt, t_lineinfo *lineInfo, t_stats *pStats)
 {
  int linePos, i;
+ BOOL isOK, isEnd, tmpNick1S = FALSE, tmpNick2S = FALSE;
  t_user_entry *tmpUser;
- char tmpStr[SET_MAX_NICKLEN + 1];
- int iHours, iSeconds, iMinutes;
+ char	tmpStr[SET_MAX_NICKLEN + 1] = "",
+ 	tmpNick1[SET_MAX_NICKLEN + 1],
+ 	tmpNick2[SET_MAX_NICKLEN + 1],
+ 	tmpDest, c;
 
  /* Initialize */
  linePos = 0;
- iHours = iMinutes = iSeconds = -1;
  tmpUser = NULL;
-
-
+ isOK = TRUE;
+ 
  /* Parse the line via format-string */
- while (*fmt)
+ while (*fmt && isOK)
  {
  if (*fmt == '%')
 	{
 	switch (*(++fmt)) {
+	/* Generic matching */
 	case '?':
 		/* Match anything */
 		fmt++;
-		linePos++;
+		if (inLine[linePos])
+			linePos++;
+			else
+			isOK = FALSE;
 		break;
 
 	case '*':
 		/* Match anything until next char */
 		fmt++;
-		
-		/* Search for it */
-		while (inLine[linePos] && (inLine[linePos] != *fmt))
-			linePos++;
-
+		while (inLine[linePos] && (inLine[linePos] != *fmt)) linePos++;
 		break;
 		
 	case '@':
 		/* Match irssi style optional '@|+| ' */
-		if ((inLine[linePos] == '@') || (inLine[linePos] == '+') || isspace(inLine[linePos]))
+		fmt++;
+		if (!inLine[linePos]) isOK = FALSE;
+		if ((inLine[linePos] == '@') ||
+			(inLine[linePos] == '+') ||
+			th_isspace(inLine[linePos]))
 			linePos++;
-		fmt++;
 		break;
 
-	/* Timestamp */
-	case 'H':
-		/* Hours */
-		iHours = parse_int(inLine, 2, &linePos);
-		fmt++;
-		break;
+	/* Timestamps */
+	case 'H': lineInfo->tHours = parse_int(inLine, 2, &linePos); fmt++; break;
+	case 'M': lineInfo->tMinutes = parse_int(inLine, 2, &linePos); fmt++; break;	
+	case 'S': lineInfo->tSeconds = parse_int(inLine, 2, &linePos); fmt++; break;
 
-	case 'M':
-		/* Minutes */
-		iMinutes = parse_int(inLine, 2, &linePos);
-		fmt++;
-		break;
+	case 'Y': lineInfo->dYear = parse_int(inLine, 4, &linePos); fmt++; break;
 		
-	case 'S':
-		/* Seconds */
-		iSeconds = parse_int(inLine, 2, &linePos);
+	case 'y':
+		/* 2-digit year */
+		i = parse_int(inLine, 2, &linePos);
+		if (i < 70)
+			i += 2000;
+			else
+			i += 1900;
+
+		lineInfo->dYear = i;
 		fmt++;
 		break;
 
-	/* Nick */
+	case 'd': lineInfo->dDay = parse_int(inLine, 2, &linePos); fmt++; break;
+	case 'j': lineInfo->dMonth = parse_int(inLine, 2, &linePos); fmt++; break;
+
+
+	/* Special matches */
 	case 'n':
-		/* Get ending character */
+	case 'N':
+		/* Nick */
+		tmpDest = *fmt;
 		fmt++;
-		
+
 		/* Find the start of the nick */
 		th_findnext(inLine, &linePos);
 
 		/* Get the nick to temp buffer */
-		i = 0;
-		while ((inLine[linePos]) && (inLine[linePos] != *fmt) && (i < SET_MAX_NICKLEN))
-			tmpStr[i++] = inLine[linePos++];
+		i = 0; isEnd = FALSE;
 
+		c = inLine[linePos];
+		if (!th_isalpha(c) && !th_isspecial(c)) isOK = FALSE;
+		
+		while (isOK && !isEnd)
+			{
+			c = inLine[linePos];
+			if (!c || (c == *fmt) || th_isspace(c) || (i >= SET_MAX_NICKLEN))
+				isEnd = TRUE;
+				else
+				{
+				if (th_isalpha(c) || th_isdigit(c) || th_isspecial(c) || (c == '-'))
+					tmpStr[i++] = inLine[linePos++];
+					else
+					isOK = FALSE;
+				}
+			}
+			
 		tmpStr[i++] = 0;
 		
-		/* Find user or add new */
-		if (i > 0)
-			tmpUser = parse_newuser(pStats, tmpStr, autoAlias);
+		while (inLine[linePos] && th_isspace(inLine[linePos]) && (inLine[linePos] != *fmt)) linePos++;
+		if (inLine[linePos] != *fmt) isOK = FALSE;
 
-		/* And neext .. */
+		/* Find user or add new */
+		if (isOK && (i > 0))
+			switch (tmpDest) {
+			case 'n': tmpNick1S = TRUE; strcpy(tmpNick1, tmpStr); break;
+			case 'N': tmpNick2S = TRUE; strcpy(tmpNick2, tmpStr); break;
+			}
 		break;
-	
+
+	case 'm':
+		/* Mode */
+		fmt++;
+		while (inLine[linePos] && (inLine[linePos] != *fmt)) linePos++;
+		break;
+
+	case 'c':
+		/* Channel */
+		fmt++;
+		while (inLine[linePos] && (inLine[linePos] != *fmt)) linePos++;
+		break;
+
+	case 't':
+		/* Text */
+		fmt++;
+		i = 0;
+		while (inLine[linePos] && (inLine[linePos] != *fmt) && (i < SET_MAX_BUF))
+			lineInfo->pText[i++] = inLine[linePos++];
+
+		lineInfo->pText[i++] = 0;
+		break;
+
 	/* Error */
 	default:
-		fprintf(stderr, "parse_msg_public: Syntax error in format-string '%s'\n", fmt);
+		FERR("Syntax error in format-string '%s'\n", fmt);
 		return -1;
 	}
 	} else {
 	/* Check matches */
 	if (*fmt != inLine[linePos])
-		return 1;
+		isOK = FALSE;
 
 	fmt++;
 	linePos++;
@@ -375,63 +437,102 @@ int parse_public(char *inLine, char *fmt, t_stats *pStats, BOOL autoAlias)
  } /* while(*fmt) */
 
 
- /* Check if we got user for this ... */
- if (tmpUser)
- {
- t_int	nWords, nQuestions, nYelling,
- 	nHappyFaces, nSadFaces;
+ if (isOK)
+ 	{
+ 	if (tmpNick1S)
+ 		lineInfo->pUser = parse_newuser(pStats, tmpNick1);
 
- int tmpPos;
+	if (tmpNick2S)
+		lineInfo->pUser2 = parse_newuser(pStats, tmpNick2);
+ 	}
+
+ return !isOK;
+}
+
+
+t_user_entry *parse_public(char *infLine, char *fmt, t_stats *pStats)
+{
+ t_lineinfo lineInfo;
+ t_int	nWords, nQuestions, nYelling;
+ int linePos;
  BOOL isWord;
 
+ /* Try to parse the line */
+ if (parse_generic(infLine, fmt, &lineInfo, pStats))
+ 	return NULL;
 
+ /* If the text is empty, we don't need to analyze it */
+ if (!lineInfo.pText[0])
+ 	return lineInfo.pUser;
+ 
  /* Detect URLs */
- if (strstr(&inLine[linePos], "http://") != NULL)
-	{
+ if (strstr(lineInfo.pText, "http://") != NULL)
+ 	{
 	/* Add the URL in list */
-	tmpUser->nURLs++;
+	lineInfo.pUser->nURLs++;
 	}
 
  /* Statisticize the actual public message-line */
- tmpPos = linePos;
+ linePos = 0;
  isWord = FALSE;
 
- nHappyFaces = nSadFaces = nQuestions = nYelling = nWords = 0;
+ nQuestions = nYelling = nWords = 0;
 
- while (inLine[linePos])
- 	{
- 	if (isWord && isspace(inLine[linePos]))
- 		{
- 		nWords++;
- 		isWord = FALSE;
- 		} else
- 	if ((!isWord) && !isspace(inLine[linePos]))
+ while (lineInfo.pText[linePos])
+	{
+	if (isWord && th_isspace(lineInfo.pText[linePos]))
+		{
+		nWords++;
+		isWord = FALSE;
+		} else
+	if ((!isWord) && !th_isspace(lineInfo.pText[linePos]))
 		isWord = TRUE;
 
-	if (isupper(inLine[linePos]))
-		tmpUser->nCaps++;
+	if (th_isupper(lineInfo.pText[linePos]))
+		lineInfo.pUser->nCaps++;
 
-	switch (inLine[linePos]) {
+	switch (lineInfo.pText[linePos]) {
+	case '=':
 	case ':':
 	case ';':
-		switch (inLine[linePos + 1]) {
+		switch (lineInfo.pText[linePos + 1]) {
 		case ')': /* :) */
 		case 'D': /* :D */
 		case 'P': /* :P */
 		case '>': /* :> */
 		case ']': /* :] */
-			nHappyFaces++;
+			lineInfo.pUser->nHappiness++;
 			break;
 	
 		case '(': /* :( */
 		case '[': /* :[ */
 		case '/': /* :/ */
 		case 'I': /* :I */
-			nSadFaces++;
+			lineInfo.pUser->nHappiness--;
 			break;
 		}
 		break;
 
+	case '(':
+	case '<':
+		switch (lineInfo.pText[linePos + 1]) {
+		case ':':
+		case ';':
+			lineInfo.pUser->nHappiness++;
+			break;
+		}
+		break;
+			
+	case ')':
+	case '>':
+		switch (lineInfo.pText[linePos + 1]) {
+		case ':':
+		case ';':
+			lineInfo.pUser->nHappiness--;
+			break;
+		}
+		break;
+			
 	case '!':
 		nYelling++;
 		break;
@@ -441,32 +542,145 @@ int parse_public(char *inLine, char *fmt, t_stats *pStats, BOOL autoAlias)
 		break;
 	}
 
-
- 	tmpUser->nChars++;
-
- 	linePos++;
- 	}
+	lineInfo.pUser->nChars++;
+	linePos++;
+	}
 
  /* Add to user's stats */
- if (nYelling) tmpUser->nYelling++;
- if (nQuestions) tmpUser->nQuestions++;
- if (nHappyFaces) tmpUser->nHappyFaces++;
- if (nSadFaces) tmpUser->nSadFaces++;
+ if (nYelling) lineInfo.pUser->nYelling++;
+ if (nQuestions) lineInfo.pUser->nQuestions++;
  
- tmpUser->nWords += nWords;
- tmpUser->nPublics++;
+ lineInfo.pUser->nWords += nWords;
+ lineInfo.pUser->nPublics++;
 
- if ((iHours >= 0) && (iHours < SET_HOURS_DAY))
- 	{
-	tmpUser->nWordsPerHour[iHours] += nWords;
-	tmpUser->nPublicsPerHour[iHours]++;
+ if ((lineInfo.tHours >= 0) && (lineInfo.tHours < SET_HOURS_DAY))
+	{
+	lineInfo.pUser->nWordsPerHour[lineInfo.tHours] += nWords;
+	lineInfo.pUser->nPublicsPerHour[lineInfo.tHours]++;
 
-	if (tmpUser->nWords >= (tmpUser->nWordsPerHour[iHours] / (tmpUser->nPublicsPerHour[iHours]+1)))
+	if (lineInfo.pUser->nWords >=
+		(lineInfo.pUser->nWordsPerHour[lineInfo.tHours] /
+		(lineInfo.pUser->nPublicsPerHour[lineInfo.tHours]+1)))
 		{
-		if ((!tmpUser->sComment) || (random() < (RAND_MAX / 3)))
-			th_strcpy(&tmpUser->sComment, &inLine[tmpPos]);
+		if ((!lineInfo.pUser->sComment) || (random() < (RAND_MAX / 3)))
+		if (strlen(lineInfo.pText) > SET_MIN_COMMENT_LEN)
+			th_strcalloc(&lineInfo.pUser->sComment, lineInfo.pText);
+		}
+	}
+
+ /* Done, ok. */ 
+ return lineInfo.pUser;
+}
+
+
+t_user_entry *parse_misc(char *infLine, char *fmt, t_stats *pStats)
+{
+ t_lineinfo lineInfo;
+
+ /* Try to parse the line */
+ if (parse_generic(infLine, fmt, &lineInfo, pStats))
+ 	return NULL;
+
+ /* Done, ok. */ 
+ return lineInfo.pUser;
+}
+
+
+int parse_kick(char *infLine, char *fmt, t_stats *pStats)
+{
+ t_lineinfo lineInfo;
+
+ /* Try to parse the line */
+ if (parse_generic(infLine, fmt, &lineInfo, pStats))
+ 	return -1;
+
+ /* Add to user's stats */
+ lineInfo.pUser->nGotKicked++;
+ lineInfo.pUser2->nKicks++;
+ 
+ /* Done, ok. */ 
+ return 0;
+}
+
+
+int parse_nickchange(char *infLine, char *fmt, t_stats *pStats, BOOL autoFollow, BOOL autoHeur)
+{
+ t_lineinfo lineInfo;
+ int i;
+ 
+ /* Try to parse the line */
+ if (parse_generic(infLine, fmt, &lineInfo, pStats))
+ 	return -1;
+
+ /* Let's see if we can autofollow the nick-changes */
+ if (autoFollow && (lineInfo.pUser != lineInfo.pUser2))
+ {
+NICKDEBUG("['%s' -> '%s'] -- ", lineInfo.pUser->userHandle, lineInfo.pUser2->userHandle);
+ if (lineInfo.pUser->isManaged && !lineInfo.pUser2->isManaged)
+	{
+NICKDEBUG("'%s' is alias to '%s'\n", lineInfo.pUser2->userHandle, lineInfo.pUser->userHandle);
+	nick_change(pStats->nickList, lineInfo.pUser2, lineInfo.pUser);
+	user_delete(&pStats->userList, lineInfo.pUser2);
+	user_free(lineInfo.pUser2);
+	lineInfo.pUser->nNickChanges++;
+	} else
+ if (!lineInfo.pUser->isManaged && lineInfo.pUser2->isManaged)
+ 	{
+NICKDEBUG("'%s' is alias to '%s'\n", lineInfo.pUser->userHandle, lineInfo.pUser2->userHandle);
+
+	nick_change(pStats->nickList, lineInfo.pUser, lineInfo.pUser2);
+	user_delete(&pStats->userList, lineInfo.pUser);
+	user_free(lineInfo.pUser);
+	lineInfo.pUser2->nNickChanges++;
+ 	} else
+ if (autoHeur)
+ 	{
+ 	/*
+ 	 * Let's try to determine the "real" user with simple heuristics
+ 	 */
+NICKDEBUG("guessing... %i, %i",autoHeur,autoHeur);
+
+	i = 0;
+ 	if (strlen(lineInfo.pUser->userHandle) < strlen(lineInfo.pUser2->userHandle))
+		i--;
+		else
+		i++;
+
+	if (th_strmatch(lineInfo.pUser2->userHandle, lineInfo.pUser->userHandle))
+		i--;
+		
+	if (th_strmatch(lineInfo.pUser->userHandle, lineInfo.pUser2->userHandle))
+		i++;
+		
+	if (th_strmatch(lineInfo.pUser2->userHandle, "*^*") || th_strmatch(lineInfo.pUser2->userHandle, "*_*"))
+		i -= 2;
+
+	if (th_strmatch(lineInfo.pUser->userHandle, "*^*") || th_strmatch(lineInfo.pUser->userHandle, "*_*"))
+		i += 2;
+
+	if (i <= 0)
+		{
+NICKDEBUG("'%s' is alias to '%s'\n", lineInfo.pUser2->userHandle, lineInfo.pUser->userHandle);
+
+		nick_change(pStats->nickList, lineInfo.pUser2, lineInfo.pUser);
+		user_delete(&pStats->userList, lineInfo.pUser2);
+		user_free(lineInfo.pUser2);
+		lineInfo.pUser->nNickChanges++;
+		lineInfo.pUser->isManaged = TRUE;
+		} else {
+NICKDEBUG("'%s' is alias to '%s'\n", lineInfo.pUser->userHandle, lineInfo.pUser2->userHandle);
+
+		nick_change(pStats->nickList, lineInfo.pUser, lineInfo.pUser2);
+		user_delete(&pStats->userList, lineInfo.pUser);
+		user_free(lineInfo.pUser);
+		lineInfo.pUser2->nNickChanges++;
+		lineInfo.pUser2->isManaged = TRUE;
 		}
  	}
+ } else {
+ /* Update the stats */
+ lineInfo.pUser->nNickChanges++;
+ lineInfo.pUser2->nNickChanges++; 
  }
  
  /* Done, ok. */ 
@@ -474,18 +688,19 @@ int parse_public(char *inLine, char *fmt, t_stats *pStats, BOOL autoAlias)
 }
 
 
-
 /*
  * A generic logfile parser
  */
-int parse_log(char *logFilename, FILE *inFile, t_stats *pStats, t_logformat *logFmt, t_config *pCfg)
+int parse_log(FILE *inFile, t_stats *pStats, t_logformat *logFmt, t_config *pCfg)
 {
- BOOL autoAlias;
+ BOOL autoFollow, autoHeur;
  char inLine[SET_MAX_BUF + 1];
  int lineNum, linePos;
-
+ t_user_entry *tmpUser;
+ 
  /* Get configuration options */
- autoAlias = th_config_get_bool(pCfg, CFG_GEN_AUTO_ALIAS_NICKS, FALSE);
+ autoFollow = th_config_get_bool(pCfg, CFG_GEN_AUTO_FOLLOW_NICKS, TRUE);
+ autoHeur = th_config_get_bool(pCfg, CFG_GEN_AUTO_FOLLOW_HEURISTIC, FALSE);
 
  /* Initial stats */
  pStats->nLogFiles++;
@@ -494,7 +709,10 @@ int parse_log(char *logFilename, FILE *inFile, t_stats *pStats, t_logformat *log
  lineNum = 0;
  while (fgets(inLine, sizeof(inLine), inFile) != NULL)
  {
- pStats->nChars += strlen(inLine);
+ linePos = strlen(inLine) - 1;
+ if (inLine[linePos] == '\n') inLine[linePos--] = 0;
+ if (inLine[linePos] == '\r') inLine[linePos--] = 0;
+ pStats->nChars += linePos;
  pStats->nLines++;
  lineNum++;
  linePos = 0;
@@ -502,9 +720,22 @@ int parse_log(char *logFilename, FILE *inFile, t_stats *pStats, t_logformat *log
  /* Check if the line is OK and what type it is */ 
  if (inLine[0])
 	{
-	if (parse_public(inLine, logFmt->fmtPublic, pStats, autoAlias) > 0)
+	if (!parse_public(inLine, logFmt->fmtPublic, pStats))
+	if (!parse_public(inLine, logFmt->fmtNotice, pStats))
 		{
-		
+		if ((tmpUser = parse_public(inLine, logFmt->fmtAction, pStats)))
+			tmpUser->nActions++;
+			else
+		if ((tmpUser = parse_public(inLine, logFmt->fmtNotice, pStats)))
+			tmpUser->nNotices++;
+			else
+		if ((tmpUser = parse_misc(inLine, logFmt->fmtJoin, pStats)))
+			tmpUser->nJoins++;
+			else
+		if (parse_kick(inLine, logFmt->fmtKick, pStats))
+		if (parse_nickchange(inLine, logFmt->fmtNickChange, pStats, autoFollow, autoHeur))
+			{
+			}
 		}
  	}
  
@@ -517,15 +748,26 @@ int parse_log(char *logFilename, FILE *inFile, t_stats *pStats, t_logformat *log
 /*
  * Userfile parser
  */
-int parse_users(char *usrFilename, t_stats *pStats, t_config *pCfg)
+void parse_pfield(char *inLine, char *tmpBuf, int bufSize, int *linePos)
+{
+ int i = 0;
+
+ while (inLine[*linePos] && (inLine[*linePos] != ':') && (i < bufSize))
+	tmpBuf[i++] = inLine[(*linePos)++];
+
+ tmpBuf[i++] = 0;
+}
+
+
+int parse_userfile(char *usrFilename, t_stats *pStats, t_config *pCfg)
 {
  char inLine[SET_MAX_BUF + 1], tmpStr[SET_MAX_BUF + 1];
  FILE *inFile;
  int i, lineNum, linePos;
- t_user_entry *tmpUser;
- t_nick_entry *tmpNick;
+ t_user_entry *tmpUser = NULL;
+ t_nick_entry *tmpNick = NULL;
+ BOOL ignoredUser;
 
- assert(pCfg);
  assert(pStats);
  
  /* Get configuration options */
@@ -544,21 +786,27 @@ int parse_users(char *usrFilename, t_stats *pStats, t_config *pCfg)
  /* Check if the line is OK and what type it is */ 
  if ((strlen(inLine) > 1) && (inLine[0] != '#'))
 	{
-	/* Get the handle */
-	i = 0;
-	while (inLine[linePos] && (inLine[linePos] != ':') && (i < SET_MAX_BUF))
-		tmpStr[i++] = inLine[linePos++];
+	/* Check if it's ignored user */
+	if (inLine[linePos] == '!')
+		{
+		linePos++;
+		ignoredUser = TRUE;
+		} else
+		ignoredUser = FALSE;
 	
-	tmpStr[i++] = 0;
+	/* Get the handle */
+	parse_pfield(inLine, tmpStr, SET_MAX_BUF, &linePos);
 
 	/* Check if next field exists */
 	if (inLine[linePos] != ':')
 		{
-		fprintf(stderr, RA_NAME ": Error in userfile '%s', line %i - missing fields.\n",
+		FERR("Error in userfile '%s', line %i - missing fields.\n",
 			usrFilename, lineNum);
 		} else {
 		/* Allocate a new user and nick records */
 		tmpUser = user_new(tmpStr);
+		tmpUser->isIgnored = ignoredUser;
+		tmpUser->isManaged = TRUE;
 		user_insert(&pStats->userList, tmpUser);
 		
 		/* Get alias nicks */
@@ -566,53 +814,48 @@ int parse_users(char *usrFilename, t_stats *pStats, t_config *pCfg)
 		while (inLine[linePos] && (inLine[linePos] != ':'))
 			{
 			/* Get one nick */
+			th_findnext(inLine, &linePos);
+
 			i = 0;
 			while (inLine[linePos] && (inLine[linePos] != ':') &&
-				(!isspace(inLine[linePos])) && (i < SET_MAX_BUF))
+				(!th_isspace(inLine[linePos])) && (i < SET_MAX_BUF))
 				tmpStr[i++] = inLine[linePos++];
 	
 			tmpStr[i++] = 0;
 
-			if (isspace(inLine[linePos]))
-				linePos++;
+			th_findnext(inLine, &linePos);
 
 			/* Add to user */
 			tmpNick = nick_new(tmpStr);
 			tmpNick->user = tmpUser;
 			
-			nick_insert(pStats->nickList, tmpNick);
+			if (nick_insert(pStats->nickList, tmpNick) != 0)
+			FERR("nick_insert() failed, hash: '%s'\n", tmpStr);
 			}
 		
 		/* Get image path */
-		if (inLine[linePos])
+		if (inLine[linePos] == ':')
 			{
 			linePos++;
-			if (inLine[linePos] != ':')
+			if ((inLine[linePos] != ':') && !th_isspace(inLine[linePos]))
 				{
-				/* Get image path */
-				i = 0;
-				while (inLine[linePos] &&
-					(inLine[linePos] != ':') &&
-					(i < SET_MAX_BUF))
-					tmpStr[i++] = inLine[linePos++];
-	
-				tmpStr[i++] = 0;
-				
 				/* Set to user record */
-				th_strcpy(&tmpUser->picPath, tmpStr);
+				parse_pfield(inLine, tmpStr, SET_MAX_BUF, &linePos);
+				
+				th_strcalloc(&tmpUser->picPath, tmpStr);
 				}
 			}
 		}
  	}
  
  } /* if (fgets()) */
-
+ 
  return 0;
 }
 
 
 /*
- * Allocates and initializes a new t_stats structure
+ * Allocates and initializes a new stats structure
  */
 t_stats *stats_new(void)
 {
@@ -629,6 +872,46 @@ t_stats *stats_new(void)
 }
 
 
+/*
+ * Deallocates stats structure
+ */
+void stats_free(t_stats *pStats)
+{
+ t_user_entry *pUser, *nUser;
+ t_nick_entry *pNick, *nNick;
+ int i;
+ assert(pStats);
+ 
+ /* Free userlist */
+ pUser = pStats->userList;
+ while (pUser)
+ 	{
+ 	nUser = pUser->pNext;
+ 	user_free(pUser);
+ 	pUser = nUser;
+ 	}
+
+ /* Free nicklist */
+ for (i = 0; i < SET_HASH_MAXINDEX; i++)
+	{
+	/* Find from linked list */
+	pNick = pStats->nickList[i];
+	while (pNick)
+		{
+		nNick = pNick->pNext;
+		nick_free(pNick);
+		pNick = nNick;
+		}
+	}
+ 
+ /* Free user index */
+ free(pStats->userIndex);
+  
+ /* Free stats structure */
+ free(pStats);
+}
+
+
 /*
  * Compare function for qsort() that compares 2 nodes for nTotalScore
  */
@@ -660,10 +943,14 @@ int stats_index(t_stats *pStats)
  
  /* Create sorted index */
  pCurr = pStats->userList;
- pStats->nUsers = 0;
+ pStats->nUsers = pStats->nIgnored = 0;
  while (pCurr)
  	{
- 	pStats->nUsers++;
+ 	if (pCurr->isIgnored)
+		pStats->nIgnored++;
+		else
+		pStats->nUsers++;
+		
  	pCurr = pCurr->pNext;
  	}
 
@@ -679,7 +966,10 @@ int stats_index(t_stats *pStats)
 	pCurr = pStats->userList;
 	while (pCurr)
 		{
-		pStats->userIndex[i++] = pCurr;
+		/* Index only, if user is unignored */
+		if (!pCurr->isIgnored)
+			pStats->userIndex[i++] = pCurr;
+
 		pCurr = pCurr->pNext;
 		}
 	}
@@ -691,7 +981,7 @@ int stats_index(t_stats *pStats)
 /*
  * Compute stats from userlist
  */
-int stats_compute(t_stats *pStats, t_config *pCfg)
+int stats_compute_rank(t_stats *pStats, t_config *pCfg)
 {
  t_user_entry *pCurr;
  int i;
@@ -701,47 +991,26 @@ int stats_compute(t_stats *pStats, t_config *pCfg)
  /* Get configuration options */
  usePisgScoring = th_config_get_bool(pCfg, CFG_GEN_USE_PISG_SCORING, FALSE);
 
- /* Initialize */
- pStats->mostStupid = pStats->mostLoud = pStats->mostActions =
- pStats->mostModes = pStats->mostKicks = pStats->mostKicked =
- pStats->mostCaps = pStats->mostHappy = pStats->mostSad =
- pStats->mostURLs = pStats->userList;
- 
  /* Go through the userlist */
  pCurr = pStats->userList;
  while (pCurr)
  	{
-	/* More loud? */
-	if (pCurr->nYelling > pStats->mostLoud->nYelling)
-		pStats->mostLoud = pCurr;
-		
-	/* More stupid */
-	if (pCurr->nQuestions > pStats->mostStupid->nQuestions)
-		pStats->mostStupid = pCurr;
-
-	/* More happy? */
-	pCurr->nHappiness = ((t_float) pCurr->nHappyFaces * (t_float) pCurr->nSadFaces) / (t_float) (pCurr->nPublics + 1);
-
-	if (pCurr->nHappiness > pStats->mostHappy->nHappiness)
-		pStats->mostHappy = pCurr;
-
-	/* More sad? */
-	if (pCurr->nHappiness < pStats->mostSad->nHappiness)
-		pStats->mostSad = pCurr;
-		
-	/* More caps per chars? */
-	if ((pCurr->nChars / (pCurr->nCaps + 1)) > (pStats->mostCaps->nChars / (pStats->mostCaps->nCaps + 1)))
-		pStats->mostCaps = pCurr;
-	
-	/* More URLs pasted? */
-	if (pCurr->nURLs > pStats->mostURLs->nURLs)
-		pStats->mostURLs = pCurr;
-
+ 	/* Check if user is ignored */
+ 	if (!pCurr->isIgnored)
+ 	{
 	/* Activity */
 	iTotalActivity = 0;
 	for (i = 0; i < SET_HOURS_DAY; i++)
 		{
-		iTotalActivity += ((t_float) pCurr->nWordsPerHour[i] * (t_float) pCurr->nPublicsPerHour[i]);
+		if (usePisgScoring)
+			pCurr->activityPercentPerHour[i] = pCurr->nPublicsPerHour[i];
+			else
+			pCurr->activityPercentPerHour[i] =
+			((t_float) pCurr->nWordsPerHour[i]) * ((t_float) pCurr->nPublicsPerHour[i]);
+
+ 		iTotalActivity += pCurr->activityPercentPerHour[i];
+
+		pStats->activityPercentPerHour[i] += pCurr->activityPercentPerHour[i];
  		}
 
 	/* Compute activity-% for each hour */
@@ -750,14 +1019,11 @@ int stats_compute(t_stats *pStats, t_config *pCfg)
 		if (iTotalActivity > 0)
 			{
 			pCurr->activityPercentPerHour[i] =
-			(((t_float) pCurr->nWordsPerHour[i] * (t_float) pCurr->nPublicsPerHour[i]) / iTotalActivity) * 100.0f;
+			(pCurr->activityPercentPerHour[i] / iTotalActivity) * 100.0f;
 			} else
 			pCurr->activityPercentPerHour[i] = 0.0f;
-		
-		pStats->activityPercentPerHour[i] += pCurr->activityPercentPerHour[i];
 		}
 
-	
 	/* Compute W/P and C/W */
 	if (pCurr->nPublics > 0)
 		pCurr->nWordsPerPublic = ((t_float) pCurr->nWords / (t_float) pCurr->nPublics);
@@ -772,7 +1038,12 @@ int stats_compute(t_stats *pStats, t_config *pCfg)
 	/*
 	 * Compute total score
 	 */
-	pCurr->nTotalScore = (pCurr->nWordsPerPublic * pCurr->nCharsPerWord * pCurr->nPublics);
+	if (usePisgScoring)
+		pCurr->nTotalScore = pCurr->nPublics;
+		else
+		pCurr->nTotalScore = (pCurr->nWordsPerPublic + pCurr->nCharsPerWord) * pCurr->nPublics;
+
+	} /* isIgnored? */
 
 	/* Next node */
  	pCurr = pCurr->pNext;
@@ -783,7 +1054,7 @@ int stats_compute(t_stats *pStats, t_config *pCfg)
   */
  pStats->activityPeak = -1;
  iMaxActivity = -1;
- iTotalActivity = 0;
+ iTotalActivity = 0.0f;
  for (i = 0; i < SET_HOURS_DAY; i++)
 	{
 	iTotalActivity += pStats->activityPercentPerHour[i];
@@ -800,12 +1071,140 @@ int stats_compute(t_stats *pStats, t_config *pCfg)
 	pStats->activityPercentPerHour[i] = (pStats->activityPercentPerHour[i] / iTotalActivity) * 100.0f;
 	}
  }
-	
 
  return 0;
 }
 
 
+int stats_compute_rest(t_stats *pStats, t_config *pCfg)
+{
+ t_user_entry *pCurr;
+ t_int statMax, nUser;
+ 
+ /* Get configuration options */
+ if (th_config_get_bool(pCfg, CFG_GEN_STAT_ONLY_LISTED, FALSE))
+ 	{
+	statMax = th_config_get_int(pCfg, CFG_GEN_SHOWMAX, CFG_GEN_SHOWMAX_DEF);
+	if (statMax > pStats->nUsers)
+		statMax = pStats->nUsers;
+	} else
+	statMax = pStats->nUsers;
+ 
+ /* Initialize */
+ nUser = 0;
+ while (pStats->userIndex[nUser]->isIgnored) nUser++;
+ 
+ pStats->mostStupid = pStats->mostLoud = pStats->mostActions =
+ pStats->mostModes = pStats->mostKicks = pStats->mostKicked =
+ pStats->mostCaps = pStats->mostHappy = pStats->mostSad =
+ pStats->mostURLs = pStats->mostJoins = pStats->userIndex[nUser];
+
+ /* Go through the userlist */
+ for (nUser = 0; nUser < statMax; nUser++)
+ 	{
+ 	pCurr = pStats->userIndex[nUser];
+
+ 	/* Check if user is ignored */
+ 	if (!pCurr->isIgnored)
+ 	{
+	/* More stupid */
+	if (pCurr->nQuestions > pStats->mostStupid->nQuestions)
+		pStats->mostStupid = pCurr;
+
+	/* More loud? */
+	if (pCurr->nYelling > pStats->mostLoud->nYelling)
+		pStats->mostLoud = pCurr;
+		
+	/* More actions? */
+	if (pCurr->nActions > pStats->mostActions->nActions)
+		pStats->mostActions = pCurr;
+
+	/* More kicks? */
+	if (pCurr->nKicks > pStats->mostKicks->nKicks)
+		pStats->mostKicks = pCurr;
+
+	/* More kicked? */
+	if (pCurr->nGotKicked > pStats->mostKicked->nGotKicked)
+		pStats->mostKicked = pCurr;
+
+	/* More caps per chars? */
+	if (pCurr->nCaps > 0)
+		pCurr->nCapsPercent = ((t_float) pCurr->nCaps / (t_float) pCurr->nChars) * 100.0f;
+
+	if (pCurr->nCapsPercent > pStats->mostCaps->nCapsPercent)
+		pStats->mostCaps = pCurr;
+	
+	/* More happy? */
+	if (pCurr->nHappiness > pStats->mostHappy->nHappiness)
+		pStats->mostHappy = pCurr;
+
+	/* More sad? */
+	if (pCurr->nHappiness < pStats->mostSad->nHappiness)
+		pStats->mostSad = pCurr;
+		
+	/* More URLs pasted? */
+	if (pCurr->nURLs > pStats->mostURLs->nURLs)
+		pStats->mostURLs = pCurr;
+
+	/* More Joins? */
+	if (pCurr->nJoins > pStats->mostJoins->nJoins)
+		pStats->mostJoins = pCurr;
+
+	} /* isIgnored? */
+
+ 	}
+
+ return 0;
+}
+
+
+void output_userfile(FILE *f, t_stats *pStats)
+{
+ t_user_entry *pCurr;
+ t_nick_entry *pNick;
+ int i;
+
+ fprintf(f, "# Userfile for " RA_NAME " (" RA_FULLNAME ")\n");
+ 
+ pCurr = pStats->userList;
+ while (pCurr)
+ 	{
+ 	if (pCurr->isIgnored)
+ 		fprintf(f, "!");
+ 	
+ 	fprintf(f, "%s:", pCurr->userHandle);
+ 
+ 	for (i = 0; i < SET_HASH_MAXINDEX; i++)
+		{
+ 		/* Find from linked list */
+		pNick = pStats->nickList[i];
+		while (pNick)
+			{
+			if (pNick->user == pCurr)
+			fprintf(f, "%s ", pNick->nick);
+			pNick = pNick->pNext;
+			}
+		}
+
+	fprintf(f, ":");
+
+	if (pCurr->picPath)
+		fprintf(f, "%s:", pCurr->picPath);
+		else
+	if (pCurr->linkURL)
+		fprintf(f, ":");
+
+	if (pCurr->linkURL)
+		fprintf(f, "%s", pCurr->linkURL);
+	fprintf(f, "\n");
+
+	pCurr = pCurr->pNext;
+	}
+}
+
+
+
+
 /*
  * NOTICE: Since this utility is designed to be "one pass/shot" program,
  * we don't free any memory used here. If you take this code, remember it
@@ -834,7 +1233,8 @@ int main(int argc, char *argv[])
  myStats = stats_new();
  if (myStats == NULL)
  	{
- 	fprintf(stderr, RA_NAME ": Could not allocate memory for statistics!\n");
+ 	FERR("Could not allocate memory for statistics!\n");
+ 	return -11;
  	}
  
  /*
@@ -846,8 +1246,8 @@ int main(int argc, char *argv[])
 
  if (nsourceFileNames <= 0)
  	{
- 	fprintf(stderr, RA_NAME ": No input files specified!\n");
- 	exit(3);
+ 	FERR("No input files specified!\n");
+ 	return 0;
  	}
 
  /*
@@ -855,18 +1255,18 @@ int main(int argc, char *argv[])
   */
  if (nconfigFileNames <= 0)
  	{
- 	fprintf(stderr, RA_NAME ": No configuration file(s) specified.\n");
+ 	FERR("No configuration file(s) specified.\n");
  	}
 
  for (i = 0; i < nconfigFileNames; i++)
 	{
 	if (nverbosity >= 1)
-	fprintf(stderr, RA_NAME ": Configuration '%s'\n", configFileNames[i]);
+	FERR("Configuration '%s'\n", configFileNames[i]);
 
 	if (th_config_read(configFileNames[i], &myConfig) < 0)
 	 	{
-	 	fprintf(stderr, RA_NAME ": Error reading configuration file.\n");
-	 	exit(4);
+	 	FERR("Error reading configuration file.\n");
+	 	return -12;
 	 	}
 	}
 
@@ -874,28 +1274,28 @@ int main(int argc, char *argv[])
   * Parse the users file
   */
  if (userFileName)
-	parse_users(userFileName, myStats, myConfig);
+	parse_userfile(userFileName, myStats, myConfig);
 	else
-	parse_users(th_config_get_str(myConfig, CFG_GEN_USER_FILE, NULL), myStats, myConfig);
+	parse_userfile(th_config_get_str(myConfig, CFG_GEN_USER_FILE, NULL), myStats, myConfig);
+
  
  /*
   * Read the source-file(s)
   */
  if (nverbosity >= 1)
- fprintf(stderr, RA_NAME ": Parsing %d sources. Please wait...\n", nsourceFileNames);
+ FERR("Parsing %d sources. Please wait...\n", nsourceFileNames);
 
  for (i = 0; i < nsourceFileNames; i++)
 	{
 	/* Try to open the logfile */
 	if ((myFile = fopen(sourceFileNames[i], "ra")) == NULL)
 		{
-		fprintf(stderr, RA_NAME ": Error opening input file '%s' (%s)\n", sourceFileNames[i], strerror(errno));
+		FERR("Error opening input file '%s' (%s)\n", sourceFileNames[i], strerror(errno));
 		return -1;
 		}
 
 	/* Parse with selected parser */
-	iResult = parse_log(sourceFileNames[i],
-		myFile, myStats,
+	iResult = parse_log(myFile, myStats,
 		&inputFormats[sourceFileFormats[i]],
 		myConfig);
 
@@ -915,7 +1315,7 @@ int main(int argc, char *argv[])
 	fclose(myFile);
 	if (iResult < 0)
 		{
-		fprintf(stderr, RA_NAME ": Error #%i reading file (%s)\n", iResult, strerror(errno));
+		FERR("Error #%i reading file (%s)\n", iResult, strerror(errno));
 		return 2;
 		}
 	}
@@ -924,13 +1324,14 @@ int main(int argc, char *argv[])
 	fprintf(stderr, "\n"); 
 
  /*
-  * Calculate statistics
+  * Calculate rank and stats for sorting
   */
  if (nverbosity >= 1)
- fprintf(stderr, RA_NAME ": Computing statistics...\n");
- if (stats_compute(myStats, myConfig) < 0)
+ FERR("Computing statistics...\n");
+
+ if (stats_compute_rank(myStats, myConfig) < 0)
  	{
- 	fprintf(stderr, RA_NAME ": Error while computing statistics!\n");
+ 	FERR("Error while computing rankings!\n");
  	return -10;
  	}
 
@@ -939,28 +1340,50 @@ int main(int argc, char *argv[])
   */
  if (stats_index(myStats) < 0)
  	{
- 	fprintf(stderr, RA_NAME ": Error while indexing userlist!\n");
+ 	FERR("Error while indexing userlist!\n");
  	return -9;
  	}
 
-
  /* Sort the indexes by score */
- fprintf(stderr, RA_NAME ": %li nicks/handles, sorting...\n", myStats->nUsers);
+ if (nverbosity >= 1)
+ FERR("%li users (%li ignored), sorting...\n",
+ 	myStats->nUsers, myStats->nIgnored);
 
  qsort(myStats->userIndex, myStats->nUsers, sizeof(t_user_entry *), stats_index_cmp);
 
+ /*
+  * Compute rest of statistics
+  */
+ if (nverbosity >= 1)
+ FERR("Computing more statistics...\n");
+ if (stats_compute_rest(myStats, myConfig) < 0)
+ 	{
+ 	FERR("Error while computing statistics!\n");
+ 	return -10;
+ 	}
+
+
+ /*
+  * Check number of users
+  */
+ if (myStats->nUsers <= 0)
+ 	{
+ 	FERR("No users found? Check that you really specified correct log-format!\n");
+ 	return -10;
+ 	}
 
  /* Calculate time */
  time(&myTime2);
  myStats->nTimeElapsed = (myTime2 - myTime1);
  
  /* Open output-file */
- fprintf(stderr, RA_NAME ": Using %s\n", outputFormats[setOutputFormat].ofDescription);
+ if (nverbosity >= 1)
+ FERR("Using %s\n", outputFormats[setOutputFormat].ofDescription);
 
  if (destFileName == NULL) myFile = stdout; else
  if ((myFile = fopen(destFileName, "wa")) == NULL)
  	{
- 	fprintf(stderr, RA_NAME ": Error opening output file '%s' (%s)\n", destFileName, strerror(errno));
+ 	FERR("Error opening output file '%s' (%s)\n", destFileName, strerror(errno));
  	return -1;
  	}
 
@@ -973,30 +1396,30 @@ int main(int argc, char *argv[])
   * OK! Show final stats
   */
  if (nverbosity >= 1)
- {
- fprintf(stderr,
- 	RA_NAME ": %ld lines in %ld logfile(s), total of %1.2f MB\n",
+ FERR("%ld lines in %ld logfile(s), total of %1.2f MB\n",
  	myStats->nLines,
  	myStats->nLogFiles,
  	((t_float) myStats->nChars) / (1024.0f*1024.0f)
  	);
- }
   
  if (iResult == 0)
  	{
- 	if (nverbosity >= 1)
-	fprintf(stderr, RA_NAME ": Done.");
+	if (nverbosity >= 1)
+	FERR("Done. Time elapsed %ld hours, %ld minutes and %ld seconds.\n",
+ 		(myStats->nTimeElapsed / (60*60)),
+		(myStats->nTimeElapsed % (60*60)) / 60, 
+		(myStats->nTimeElapsed % (60*60)) % 60
+		);
 	} else
-	fprintf(stderr, RA_NAME ": Error in output! Return code #%i. ", iResult);
+	FERR("Error in output! Return code #%i.\n", iResult);
 
- if (nverbosity >= 1)
- { 
- fprintf(stderr, " Time elapsed %ld hours, %ld minutes and %ld seconds.\n",
- 	(myStats->nTimeElapsed / (60*60)),
-	(myStats->nTimeElapsed % (60*60)) / 60, 
-	(myStats->nTimeElapsed % (60*60)) % 60
-	);
- }
+
+ /*
+  * Free allocate memory
+  */
+ /*output_userfile(stdout, myStats);*/
+ stats_free(myStats);
+ th_config_free(myConfig);
  
  return 0;
 }
