@@ -7,6 +7,14 @@
  */
 #include "out_formats.h"
 
+/*
+ * Functions
+ */
+int output_xhtml(FILE *, t_stats *, t_config *);
+int output_xhtmls(FILE *, t_stats *, t_config *);
+
+
+
 /*
  * List of output formats
  */
