@@ -12,13 +12,6 @@
 #include "fisg.h"
 #include "th_config.h"
 
-/*
- * Functions
- */
-int output_xhtml(FILE *, t_stats *, t_config *);
-int output_xhtmls(FILE *, t_stats *, t_config *);
-
-
 /*
  * Typedefs
  */
