@@ -13,18 +13,18 @@
 /*
  * Type definations
  */
-enum {
+enum ITEM_TYPE {
 	ITEM_STRING = 1,
 	ITEM_INT,
 	ITEM_BOOL
 };
 
-typedef struct tconfiginode {
-	char	*itemName;
-	int	itemType;
-	void	*itemData;
-	
-	struct tconfiginode *pNext, *pPrev;
+typedef struct tconfignode {
+	char	*itemName;	/* Config item name */
+	int	itemType;	/* Type of the item */
+	void	*itemData;	/* Dynamically allocated contents / value */
+
+	struct tconfignode *pNext, *pPrev;
 } t_config_item;
 
 
@@ -38,6 +38,7 @@ typedef struct {
  */
 int	th_config_read(char *, t_config **);
 /*int	th_config_save(char *, t_config *);*/
+void	th_config_free(t_config *);
 
 int	th_config_get_int(t_config *, char *, int);
 char 	*th_config_get_str(t_config *, char *, char *);
