@@ -46,7 +46,12 @@ MCONFIG="defaults.cfg"
 ### End of user configurable parts
 ############################################################
 for ICHANNEL in $MCHANNELS; do
-	$INAME=`echo $ICHANNEL|sed "s/^[#!&]//"`
-	$MFISG -c $MCONFIG -c $INAME.cfg $MLOGPATH/ -o "$MHTMLPATH/$MHTMLPREFIX$INAME$MHTMLSUFFIX"
+	$INAME=`echo $ICHANNEL|sed "s/^[#!&]//"`.cfg
+	if test -f $INAME; then
+	COPT="-c $INAME"
+	else
+	COPT=""
+	fi
+	$MFISG -c $MCONFIG $COPT $MLOGPATH/ -o "$MHTMLPATH/$MHTMLPREFIX$INAME$MHTMLSUFFIX"
 done
 
